//
//  SXEffect.h
//  SXVideoEnging
//
//  Created by Yin Xie on 2020/3/31.
//  Copyright © 2020 Zhiqiang Li. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN
typedef enum : NSUInteger {
     SXColorSettingsBrightness     = 1 << 0,
     SXColorSettingsContrast       = 1 << 1,
     SXColorSettingsSaturation     = 1 << 2,
     SXColorSettingsSharpen        = 1 << 3,
     SXColorSettingsHighlight      = 1 << 4,
     SXColorSettingsShadow         = 1 << 5,
     SXColorSettingsExposure       = 1 << 6,
     SXColorSettingsHue            = 1 << 7
} SXColorSettings;

typedef enum : NSUInteger {
    SXEffectTypeTransition,
    SXEffectTypeTextBubble,
    SXEffectTypeTextStyle,
    SXEffectTypeTextAnimation,
    SXEffectTypeFilter,
    SXEffectTypeAnimation,
    SXEffectTypeVideoEffect,
    SXEffectTypeTrackAnimation,
    SXEffectTypeColorAdjust
} SXEffectType;

typedef enum : NSUInteger {
    SXEffectTimeFollowTypeNone,   //以轨道in point为基准，start time为相对与轨道in point的时间
    SXEffectTimeFollowTypeFollowStart, //类似入场，start time始终为0
    SXEffectTimeFollowTypeFollowEnd, //类似出场，效果与轨道一同结束
    SXEffectTimeFollowTypeFollowWholeProcess //效果时间以轨道时间保持一致
}SXEffectTimeFollowType;

/**
 * 特效，被应用在轨道或组上产生效果
 * @warning 注意所有的对象对比是否相等时请使用id对比，引用可能会不同
*/
@interface SXEffect : NSObject
//开始时间是相对与轨道in point的时间
@property (nonatomic, assign) double startTime;
//time 时长是相对与轨道in point的时长，不得超过轨道时长
@property (nonatomic, assign) double duration;

/**
 * 效果类型
 * @return  效果类型
 */
- (SXEffectType)type;

/**
 * 效果的唯一id
 * @return uuid
 */
- (NSString *)getEffectId;

/**
 * 如果有resource，返回资源原时长
 * @return 资源原时长
 */
- (double)resourceDuration;

/**
 * 设置此效果是跟随轨道时间方式，默认为不跟随轨道时间
 * @warning 跟随轨道时间可能会造成动态修改起始时间与时长，并导致设置start time或者duration失效
 * @param type 跟随轨道时间方式
 */
- (void)setEffectFollowType:(SXEffectTimeFollowType)type;

/**
 * 获取跟随轨道时间方式
 * @return 跟随轨道时间方式
 */
- (SXEffectTimeFollowType)followType;
@end

#pragma mark - SXFilterEffect
@interface SXFilterEffect : SXEffect
//滤镜的不透明度,区间为【0，1】
@property (nonatomic, assign) float alpha;

/**
 * 设置滤镜资源
 * @param resourcePath 资源路径
 * @return 设置成功返回true，否则返回false
 */
- (BOOL)setResource:(NSString *)resourcePath;

@end

#pragma mark - SXAnimationEffect
@interface SXAnimationEffect : SXEffect
//一次循环的时长，时长会转化为倍速，因此未设置动画资源时此设置无效
@property (nonatomic, assign) double durationOfOneCycle;
//动画倍速,默认一倍速
@property (nonatomic, assign) double speed;
/**
 * 设置视频动画资源
 * @param resourcePath 资源路径
 * @return 设置成功返回true，否则返回false
 */
- (BOOL)setResource:(NSString *)resourcePath;

@end

#pragma mark - SXVideoEffect
@interface SXVideoEffect : SXEffect
/**
 * 设置视频特效资源
 * @param resourcePath 资源路径
 * @return 设置成功返回true，否则返回false
 */
- (BOOL)setResource:(NSString *)resourcePath;

@end

#pragma mark - SXTextAnimationEffect
@interface SXTextAnimationEffect : SXEffect

//一次循环的时长，时长会转化为倍速，因此未设置动画资源时此设置无效
@property (nonatomic, assign) double durationOfOneCycle;
//动画倍速,默认一倍速
@property (nonatomic, assign) double speed;

/**
 * 设置文字动画资源
 * @param resourcePath 资源路径
 * @return 设置成功返回true，否则返回false
 */
- (BOOL)setResource:(NSString *)resourcePath;

@end

#pragma mark - SXTrackAnimationEffect
@interface SXTrackAnimationEffect : SXEffect
//一次循环的时长，时长会转化为倍速，因此未设置动画资源时此设置无效
@property (nonatomic, assign) double durationOfOneCycle;
//动画倍速,默认一倍速
@property (nonatomic, assign) double speed;

/**
 * 设置轨道动画资源
 * @param resourcePath 资源路径
 * @return 设置成功返回true，否则返回false
 */
- (BOOL)setResource:(NSString *)resourcePath;
@end

#pragma mark - SXColorAdjustEffect
@interface SXColorAdjustEffect : SXEffect

/**
 * 获取调色的值域
 * @param setting 调色类型
 * @return [最小值，最大值]
*/
+ (NSArray *)getValueRangeForColorSetting:(SXColorSettings)setting;

/**
 * 获取对应类型调色的值
 * @param setting 调色类型
 * @return 值
 */
- (float)getValueForSetting:(SXColorSettings)setting;


/**
 * 设置对应类型调色的值
 * @param setting 调色类型
 * @param value 值
 */
- (void)setValue:(float)value forSetting:(SXColorSettings)setting;

/**
 * 重置所有颜色调节
 */
- (void)reset;

@end
NS_ASSUME_NONNULL_END
